@@ -1,7 +1,16 @@
+"""Generates tie-break points on tied players.
+
+Tie-breaks supported:
+  * Direct Encounter
+  * Number of wins
+  * Sonneborn-Berger
+"""
+
+
 from typing import List, Dict
 
 import pandas as pd
-from pgnhelper.utility import get_encounter_score
+import pgnhelper.utility
 
 
 def num_wins(result_df: pd.DataFrame, ranking_df: pd.DataFrame) -> pd.DataFrame:
@@ -45,7 +54,7 @@ def direct_encounter(result_df: pd.DataFrame, ranking_df: pd.DataFrame, winpoint
                 for op in g.Name:
                     if p == op:
                         continue
-                    score = get_encounter_score(result_df, p, op, winpoint, drawpoint, winpointarm, losspointarm)
+                    score = pgnhelper.utility.get_encounter_score(result_df, p, op, winpoint, drawpoint, winpointarm, losspointarm)
                     s += score[0]
                     tb.update({p: {op: s}})
 
@@ -65,15 +74,17 @@ def direct_encounter(result_df: pd.DataFrame, ranking_df: pd.DataFrame, winpoint
     return ret
 
 
-def sonneborn_berger(result_df: pd.DataFrame, ranking_df: pd.DataFrame, gpe: int=1, winpoint=1.0, drawpoint=0.5) -> pd.DataFrame:
-    """Returns a dataframe with SB column for Sonneborn-Berger score.
-    1. Add all the scores of opponents that the player has defeated.
-    2. Add all the scores of opponents that the player has drawn, then divide it by 2.
-    3. Add the scores in step 1 and step 2, it will become the Sonneborn-Berger score.
+def sonneborn_berger(result_df: pd.DataFrame, ranking_df: pd.DataFrame,
+        gpe: int=1, winpoint=1.0, drawpoint=0.5) -> pd.DataFrame:
+    """Creates a dataframe with SB column for Sonneborn-Berger score.
 
-    :param result_df: A dataframe of [Round, White, Black, Result]
-    :param ranking_df: A dataframe of standing, [Name, Games, Score]
-    :param gpe: games per encounter
+    Args:
+      result_df: A dataframe of [Round, White, Black, Result].
+      ranking_df: A dataframe of standing, [Name, Games, Score].
+      gpe: games per encounter
+
+    Returns:
+      A dataframe of round-robin result table.
     """
     tb: Dict[str, int] = {}
     ret: pd.DataFrame = ranking_df.copy()
